/*
 * This file is part of vospace-file-catalog
 * Copyright (C) 2021 Istituto Nazionale di Astrofisica
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

-- View that maps node_id to os_path (relative path of file on disk)

CREATE VIEW node_os_path AS 
WITH RECURSIVE r_os_path AS (
    SELECT node_id, relative_path, COALESCE(os_name, name) AS os_path
    FROM node
    WHERE name = ''
  UNION ALL
    SELECT n.node_id, n.relative_path, p.os_path || '/' || COALESCE(os_name, name)
    FROM node n
    JOIN r_os_path p ON p.relative_path = n.parent_relative_path
    WHERE n.parent_relative_path IS NOT NULL
)
SELECT node_id, (CASE WHEN os_path = '' THEN '/' ELSE os_path END) AS os_path FROM r_os_path;

-- View that maps node_id to vos_path (path in VOSpace)

CREATE VIEW node_vos_path AS
WITH RECURSIVE r_vos_path AS (
    SELECT node_id, path, name AS vos_path
    FROM node
    WHERE name = ''
  UNION ALL
    SELECT n.node_id, n.path, p.vos_path || '/' || name
    FROM node n
    JOIN r_vos_path p ON p.path = n.parent_path
)
SELECT node_id, (CASE WHEN path = '' THEN '/' ELSE vos_path END) AS vos_path FROM r_vos_path;

-- View that combines the two previous views to obtain the os_path from the vos_path and vice versa

CREATE VIEW node_path AS
SELECT v.node_id, vos_path, os_path
FROM node_vos_path v
LEFT JOIN node_os_path o ON o.node_id = v.node_id;

-- Example query to obtain the the ltree_path from the os_path:
-- SELECT path FROM node n JOIN node_os_path o ON n.node_id = o.node_id WHERE os_path = '/curban/store';
